/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <iostream>
#include <sstream>

#include "experimenter/combo_box_spin.h"

using namespace Tk;
using namespace Experimenter;


ComboBoxSpin:: ComboBoxSpin(double climb_rate,
			    guint digits)
  : _digits(digits),
    _spin(climb_rate, digits)
{
  /* Spin button */
  add(_spin);
  
  /* Combo list data */
  _store = Gtk::ListStore::create(_columns);
  set_model(_store);
  
  //  Gtk::CellRendererText *renderer = Gtk::manage( new Gtk::CellRendererText() );
  // renderer->property_editable() = false;
  pack_start( _columns.description );
  
  //  set_cell_data_func(_renderer, sigc::mem_fun(*this, &ComboBoxSpin::cell_renderer_set_data));
  
  signal_changed().connect( sigc::mem_fun(*this, &ComboBoxSpin::combo_change_handler) );
}

void
ComboBoxSpin:: set_adjustment(Gtk::Adjustment &adj)
{
  _spin.set_adjustment(adj);
}

String
ComboBoxSpin:: get_value_description(double value)
{
  std::wostringstream sstr;
  sstr.setf(std::ios::fixed, std::ios::floatfield);
  sstr.precision(_digits);
  
  sstr << value;

  return sstr.str();
}

void
ComboBoxSpin:: combo_change_handler()
{
  set_value( get_active()->get_value(_columns.numerical_value) );
}

void
ComboBoxSpin:: add_value(double value, const String *description)
{
  Gtk::ListStore::iterator new_row( _store->append() );
  
  new_row->set_value(_columns.numerical_value, value);
  
  std::string resolved_desc;
  if (description)
    resolved_desc = *description;
  else
    resolved_desc = get_value_description(value);
  
  Glib::ustring final_desc(resolved_desc);
  new_row->set_value(_columns.description, final_desc);
}

double
ComboBoxSpin:: get_value() const
{
  return _spin.get_value();
}

void
ComboBoxSpin:: set_value(double value)
{
  _spin.set_value(value);
}
