/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkcommon/debug.h"
#include "libtkrandom/retriever_registry.h"
#include "randomness_source.h"

using namespace Tk::Experimenter;

RandomnessSource:: RandomnessSource()
{
  _store = Gtk::ListStore::create(_columns);
  set_model(_store);

  pack_start( _columns.description );
  
  /* fill data */
  Tk::Strings sources( Tk::Random::RetrieverRegistry::get_names() );
  
  bool selection_set(false);
  for (Tk::Strings::iterator iter = sources.begin();
       iter != sources.end();
       ++iter)
    {
      Gtk::ListStore::iterator new_row( _store->append() );
  
      new_row->set_value(_columns.name, wstring_to_ustring(*iter));
      
      Tk::String desc( Tk::Random::RetrieverRegistry::get_retriever_description(*iter) );
      new_row->set_value(_columns.description, wstring_to_ustring(desc));
      
      if (!selection_set && Tk::Random::RetrieverRegistry::is_true_randomness(*iter))
	{
	  set_active(new_row);
	  selection_set = true;
	}
    }
  
  if (!get_active())
    {
      set_active(0);
    }
}

Tk::String
RandomnessSource:: get_randomness_source() const
{ 
  return get_active()->get_value(_columns.name);
}
