/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "experimenter/results_store.h"
#include "experimenter/database_operation.h"

using namespace Tk::Experimenter;

ResultsStore:: ResultsStore(Gtk::Window &parent, Tk::Db::Database *database)
  : _parent_window(parent),
    _database(database),
    Gtk::ToggleToolButton(Gtk::Stock::DND_MULTIPLE)
{
  set_active(false);
}

ResultsStore:: ~ResultsStore()
{
  for (Results::iterator iter( _results.begin() );
       iter != _results.end();
       ++iter)
    delete *iter;
}

void
ResultsStore:: flush()
{
  WDEBUG_OUT << L"Flushing all experiment results..." << std::endl;
  if (_results.size() > 0)
    {
      for (Results::iterator iter( _results.begin() );
	   iter != _results.end();
	   ++iter)
	{
	  WDEBUG_OUT << L"Flushing experiment results..." << std::endl;
	  _database->insert_experiment(*iter);
	  WDEBUG_OUT << L"...flushed experiment results" << std::endl;
	}
      
      _results.clear();
      
      DatabaseOperation op(_parent_window, _database);
      op.run();
    }
  WDEBUG_OUT << L"...flushed all experiment results" << std::endl;
  
}

void
ResultsStore:: on_toggled()
{
  flush();
}

void
ResultsStore:: store_results(Tk::Db::ExperimentResult *experiment)
{
  _results.push_back(experiment);
  
  if (!get_active())
    flush();
}

