/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_MBS_H__
#define __TK_MBS_H__

#include <wchar.h>
#include <stdlib.h>
#include <cstring>
#include <string.h>

#include <glibmm.h>

#include "exceptions.h"

namespace Tk {

  wchar_t * mbstowcs_alloc(const char *string);
  char * wcstombs_alloc(const wchar_t * string);
  
  inline Glib::ustring wstring_to_ustring(const std::wstring & wstr)
    {
      char * str = wcstombs_alloc(wstr.c_str());
      Glib::ustring ustr(str);
      free(str);
      return ustr;
    }

  inline std::wstring ustring_to_wstring(const Glib::ustring & ustr)
    {
      wchar_t * str = mbstowcs_alloc(ustr.c_str());
      std::wstring wstr(str);
      free(str);
      return wstr;
    }

  inline std::string wstring_to_string(const std::wstring & wstr)
    {
      char * str = wcstombs_alloc(wstr.c_str());
      std::string sstr(str);
      free(str);
      return sstr;
    }

  inline std::wstring mbs_to_wstring(const char * cstr)
    {
      wchar_t * str = mbstowcs_alloc(cstr);
      std::wstring wstr(str);
      free(str);
      return wstr;
    }
  inline std::wstring string_to_wstring(const std::string & sstr)
    {
      return mbs_to_wstring(sstr.c_str());
    }
  
  
}; /* namespace Tk */


#endif /* __TK_MBS_H__ */
