/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2008 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DB_UI_TEMPORAL_VALUE_H__
#define __TK_DB_UI_TEMPORAL_VALUE_H__

#include <gtkmm.h>

#include "libtkdbui/value.h"

namespace Tk {
  namespace Db {
    namespace Ui {
      
      class TemporalValue : public Tk::Db::Ui::Value, public Gtk::HBox
      {
      private:
	class DateDialog : public Gtk::Dialog
	  {
	  private:
	    Gtk::Calendar  _calendar;
	  public:
	    DateDialog(unsigned day, unsigned month, unsigned year);
	    void get_date(unsigned &day, unsigned &month, unsigned &year);
	  };
	
	class TimeDialog : public Gtk::Dialog
	  {
	  private:
	    Gtk::SpinButton _hours;
	    Gtk::SpinButton _minutes;
	    Gtk::SpinButton _seconds;
	  public:
	    TimeDialog(unsigned hours, unsigned minutes, unsigned seconds);
	    void get_time(unsigned &hours, unsigned &minutes, unsigned &seconds);
	  };
	
	
	odbc::Date         _date;
	odbc::Time        *_time;
	
	Gtk::Button        _date_button;
	Gtk::Button       *_time_button;
	
	void init(const Tk::Db::Data::Type type, const odbc::Date *date, const odbc::Time *time);
	void init_widgets(const Tk::Db::Data::Type type);
	
	void update_date_display();
	void update_time_display();
	void update_sensitivity();
	
	void date_clicked();
	void time_clicked();
	
      public:
	TemporalValue(const Tk::Db::Data::Value &value);
	TemporalValue(const Tk::Db::Data::Type  &type);
	~TemporalValue();
	virtual Tk::Db::Data::Value get_value() const;
      };
      
    } /* namespace Ui */
  } /* namespace Db */
}; /* namespace Tk */

#endif /* __TK_DB_UI_TEMPORAL_VALUE_H__ */
