/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkdbui/ternary_value.h"

using namespace Tk::Db::Ui;

TernaryValue:: TernaryValue(Value *value)
  : _value(value)
{
  pack_start(_null, false, false);
  _null.set_active( _value->get_value().get_datum().is_null() );
  _null.signal_toggled().connect( sigc::mem_fun(*this, &TernaryValue::null_toggled) );
  
  pack_start(*dynamic_cast<Gtk::Widget *>(_value), true, true);
  _value->signal_value_changed().connect( sigc::mem_fun(*this, &TernaryValue::value_changed) );
  
  update_sensitivity();
}

TernaryValue::SignalValueChanged
TernaryValue:: signal_value_changed()
{
  return _signal_value_changed;
}

void
TernaryValue:: value_changed()
{
  if (_null.get_active())
    return;
  
  _signal_value_changed.emit(_value->get_value());
}

void
TernaryValue:: update_sensitivity()
{
  dynamic_cast<Gtk::Widget *>(_value)->set_sensitive( !_null.get_active() );
}

void
TernaryValue:: null_toggled()
{
  if (_null.get_active())
    {
      if (_value->get_value().get_datum().is_valid())
	_signal_value_changed.emit(Tk::Db::Data::Value(_value->get_type()));
      update_sensitivity();
    }
  else
    {
      Tk::Db::Data::Value v( _value->get_value() );
      if (v.get_datum().is_valid())
	_signal_value_changed.emit(v);
      update_sensitivity();
    }
}
