/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DECISION_TREE_H__
#define __TK_DECISION_TREE_H__

#include "libtkrandom/types.h"
#include "libtkrandom/retriever.h"
#include "libtkrandom/randomness.h"

namespace Tk {
  namespace Random {
    
    typedef unsigned Choice;
    
    class Effector
    {
    public:
      /** return false to finish deciding */
      virtual bool affect(Choice choice) = 0;
    };
    

    class DecisionTree
    {
    private:
      class DecisionState;
      
      class Node
      {
      private:
	unsigned   _bit_multiplier;
	Node      *_children[2];
	Choice    *_choice;
	
	Node (unsigned choice_count, unsigned bit_multiplier, unsigned depth, Choice &choice);
	void init(unsigned choice_count, unsigned bit_multiplier, unsigned depth, Choice &choice);
      public:
	Node (unsigned choice_count, unsigned bit_multiplier);
	~Node ();

	bool decide(Randomness &rand, unsigned bit_count = 0, unsigned bit_index = 0) const;
	Choice choose(const Node &root,
		      Randomness &rand,
		      unsigned &retries,
		      unsigned bit_count = 0,
		      unsigned bit_index = 0) const;
	static Choice choose(const Node &root,
			     Randomness &rand,
			     unsigned &retries,
			     const DecisionState &decision_state);
      };
      
      class Collector : public Tk::Random::Collector
      {
      private:
	const Node       *_root;
	Randomness       &_randomness;
	Effector         &_effector;
	unsigned          _retries;
	DecisionState    *_decision_state;
      public:
	Collector(const Node *root, Randomness &randomness, Effector &effector);
	~Collector();
	virtual bool collect(Buffer *buffer);
	unsigned get_retries() const;
      };
      
      class DecisionState
      {
      private:
	const Node *_node;
	unsigned _bit_count;
	unsigned _bit_index;
      public:
	DecisionState(const Node *node, unsigned bit_count = 0, unsigned bit_index = 0);
	const Node * get_node() const;
	unsigned get_bit_count() const;
	unsigned get_bit_index() const;
      };
      
      class PartialTraversal
      {
      private:
	DecisionState *_state;
      public:
	PartialTraversal(DecisionState *state);
	DecisionState *get_decision_state() const;
      };

      Node             *_root;
      
    public:
      static unsigned get_depth(unsigned choice_count);
      static unsigned tree_leaf_count(unsigned choice_count);
      static double expected_tries(unsigned choice_count);
      static bool deterministic(unsigned choice_count);
      
      DecisionTree (unsigned choice_count, unsigned bit_multiplier);
      ~DecisionTree ();
      
      /*** Returns the number of retries */
      unsigned decide(Retriever &retriever, Randomness &randomness, Effector &effector);
    };

    
  };
};


#endif /* __TK_DECISION_TREE_H__ */
