/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <iostream>
#include <stdexcept>

#include <curlpp/cURLpp.hpp>
#include <curlpp/Options.hpp>
#include <curlpp/Easy.hpp>
#include <utilspp/Functors.hpp>
#include <utilspp/functor/Binder.hpp>

#include "libtkcommon/exceptions.h"
#include "libtkcommon/debug.h"
#include "libtkcommon/config.h"
#include "libtkcommon/string.h"
#include "libtkrandom/hotbits.h"
#include "libtkrandom/retriever_registry.h"

using namespace Tk;
using namespace Random;

// https://www.fourmilab.ch/cgi-bin/Hotbits?nbytes=64&fmt=bin
static const String URL_BASE("https://www.fourmilab.ch/cgi-bin/Hotbits?fmt=bin&nbytes=");
static const unsigned short TIMEOUT(5000); /* milliseconds */

cURLpp::Cleanup _curl;

HotBits:: HotBits (unsigned count)
  : Block(count)
{
}

unsigned
HotBits:: get_block_size()
{
  return 1024;
}

class DataWriter
{
private:
  Buffer *_buf;
  
public:
  DataWriter(Buffer *buf)
    : _buf(buf) {}
  
  size_t write_curl_data(char *ptr, size_t size, size_t nmemb)
  {
    size_t total_size( size * nmemb );
    size_t bitchar_total( total_size / sizeof(BitChar) );
    BitChar *char_data( static_cast<BitChar *>(ptr) );
    
    for (unsigned i = 0; i < bitchar_total; ++i)
      _buf->push_back(char_data[i]);
    
    DEBUG_OUT << "Read " << total_size << " bytes from HotBits server" << std::endl;
    
    return total_size;
  }
};

Buffer *
HotBits:: get_buffer(unsigned char_count)
{
  Buffer * buf = new Buffer;
  
  cURLpp::Easy request;
  
  // Tell fourmilab what it is that's grabbing their nice data
  cURLpp::Options::UserAgent user_agent( std::string("TkResolver++ ") + PACKAGE_VERSION );
  request.setOpt(user_agent);
  
  cURLpp::Options::Url url( std::string(URL_BASE) + static_cast<std::string>(String(char_count)) );
  request.setOpt(url);
  
  DataWriter writer(buf);
  cURLpp::Options::WriteFunction write_func( utilspp::make_functor(&writer, &DataWriter::write_curl_data) );
  request.setOpt(write_func);
  
  request.perform();
  
  return buf;
}

bool
HotBitsType:: is_true_randomness()
{
  return true;
}

Retriever *
HotBitsType:: new_retriever(unsigned count)
{
  return new HotBits(count);
}
