/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <iostream>
#include <stdexcept>

#include "libtkcommon/exceptions.h"
#include "libtkcommon/debug.h"
#include "reg1.h"
#include "retriever_registry.h"

using namespace Tk::Random;

static const unsigned short USB_VENDOR_FTDI         = 0x0403;
static const unsigned short USB_PRODUCT_REG1        = 0xb5e8;
static const unsigned short USB_ENDPOINT_RANDOMNESS = 1;
static const unsigned short TIMEOUT                 = 5000; /* milliseconds */

struct usb_dev_handle * Reg1::_handle(0);
pthread_mutex_t         Reg1::_lock = PTHREAD_MUTEX_INITIALIZER;

Reg1:: Reg1(unsigned count)
  : USBDev(count, &_handle, &_lock, 0x00000081, false)
{
  open(USB_VENDOR_FTDI, USB_PRODUCT_REG1);
}

void
Reg1:: condition_data(char *data, unsigned len, char **data_out, unsigned *len_out)
{
  if (data[0] != 0x31 && data[1] != 0x60)
    throw std::runtime_error(std::string("unexpected random data prefix `")
			     + static_cast<std::string>(Tk::String(Tk::String(data[0]) + Tk::String(data[1])))
			     + "' from REG-1 device");
  
  *data_out = data + 2;
  *len_out  = len  - 2;
}

Retriever *
Reg1Type:: new_retriever(unsigned count)
{
  return new Reg1(count);
}
