/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RANDOM_RNG_H__
#define __TK_RANDOM_RNG_H__

#include <cstring>

#include "libtkrandom/retriever.h"
#include "libtkrandom/retriever_registry.h"

namespace Tk {
  namespace Random {
    
    /*** Generate a random number between 0 and an upper limit.
     *
     * This assumes that sizeof(N)/sizeof(BitChar) is an integer
     */
    template <typename N>
    N random(N upper, const Tk::String &retriever_type = "pseudo")
    {
      class Collector : public Tk::Random::Collector
      {
      private:
        const unsigned _char_count;
	BitChar       *_buffer_mem;
	unsigned       _mem_index;
      public:
	Collector(unsigned char_count)
	  : _char_count(char_count),
	    _buffer_mem( new BitChar[char_count] ),
	    _mem_index(0)
	{}
	~Collector()
	{
	  delete[] _buffer_mem;
	}
	virtual bool collect(Buffer *buffer)
	{
	  for (Buffer::iterator iter( buffer->begin() );
	       iter != buffer->end();
	       ++iter, ++_mem_index)
	    {
	      _buffer_mem[_mem_index] = *iter;
	    }
	  
	  return _mem_index < _char_count;
	}
 	N get_num() const
	{
	  N n;
	  memcpy(&n, _buffer_mem, sizeof(N));
	  return n;
	}
      };
      
      static const unsigned char_count( sizeof(N) > sizeof(BitChar) ? sizeof(N) / sizeof(BitChar) : 1 );
      
      Retriever *retriever( RetrieverRegistry::new_retriever(retriever_type, char_count) );
      Collector collector(char_count);
      retriever->retrieve(collector);
      
      N n( collector.get_num() );
      
      static const N max( std::numeric_limits<N>::max() );
      typedef long double Super;
      static const Super max_divisor( static_cast<Super>( std::numeric_limits<N>::max() ) + static_cast<Super>(1.0) );
      Super bits_ratio( static_cast<Super>(n) / max_divisor );
      Super d_value( static_cast<Super>(upper) * bits_ratio );
      return static_cast<N>(d_value);
    }
    
    template <typename N>
    N random(N lower, N upper, const Tk::String &retriever_type = "pseudo")
    {
      return lower + random<N>(upper - lower, retriever_type);
    }
  };
};


#endif /* __TK_RANDOM_RNG_H__ */
