/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RANDOM_USBDEV_H__
#define __TK_RANDOM_USBDEV_H__

#include <usb.h>

#include "libtkrandom/retriever.h"
#include "libtkrandom/retriever_registry.h"

namespace Tk {
  namespace Random {
    
    class USBDev : public Retriever
    {
    private:
      struct usb_dev_handle       **_handle_ptr;
      pthread_mutex_t              *_lock;
      int                           _endpoint;
      bool                          _check_short_read;
      void lock();
      void unlock();
    protected:
      
      USBDev(unsigned count,
	     struct usb_dev_handle **handle_ptr,
	     pthread_mutex_t *lock,
	     int endpoint,
	     bool check_short_read);
      void open(int vendor_id, int product_id);
      int control_msg(int request_type, int request, int value, int index, char *bytes, int size, int timeout);
      int bulk_read(int ep, char *bytes, int size, int timeout);
      virtual void condition_data(char *data, unsigned len, char **data_out, unsigned *len_out);
    public:
      static const unsigned short    BLOCK_SIZE;
      virtual void retrieve(Collector &collector);
    };

    class USBDevType : public RetrieverType
    {
    public:
      virtual bool is_true_randomness();
      virtual unsigned get_preferred_block_size();
      virtual Retriever * new_retriever(unsigned count) = 0;
    };
  };
  
};


#endif /* __TK_RANDOM_USBDEV_H__ */
