/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DISPLAY_H__
#define __TK_DISPLAY_H__

#include <list>
#include <vector>

#include <SDL/SDL.h>

#include "libtkcommon/pair.h"
#include "libtkresolver/sdl.h"
#include "libtkresolver/types.h"

namespace Tk {
  namespace Resolver {
    
    class Escaped {};
    
    class Display
    {
    private:
      typedef unsigned Coordinate;
      typedef std::vector<Coordinate> Coordinates;
      typedef std::list<Coordinates>  LineHistory;
      typedef std::list<SDL_Rect>     BlockList;
    
      typedef enum
      {
	Started,
	Scrolling,
	Finishing,
	Finished
      } State;
    
      const unsigned       _trace_count;
      const DirectionValue _arrow_direction;
      const Time           _total_time; /**< In milliseconds */
      
      SDL                  _sdl;

      SDL_Surface         *_display;
      SDL_Surface         *_traces;
      SDL_Surface         *_arrow;
      SDL_Rect             _arrow_dest;
      Coordinate           _horizon;
      Coordinate           _l;
      Uint32               _foreground;
      Uint32               _background;
    
      Time                 _start_time;
      State                _state;
      unsigned             _line_count;
      LineHistory          _x_history;
      Coordinate           _prev_y;
      SDL_Rect            *_prev_traces;
    
      SDL_Surface *create_surface(Uint32 flags, unsigned width, unsigned height, Uint32 fill);
      void create_traces();
      void create_arrow();
      
      Coordinates calc_start_x() const;
      int truncate_x(int x) const;
      void get_x_limits(OffsetIndex index, Coordinate &min, Coordinate &max) const;
      
      void clear_previous();
      SDL_Rect *calc_trace_dests(Coordinate y);
      void scroll_traces(const SDL_Rect *trace_dests, const Coordinates &prev_x, const Coordinates &x);
      void draw_pixels(const Line &line, const Coordinates &x, Coordinate y);
      bool draw_arrow_p(const SDL_Rect *trace_dests);
      void compose_display(const SDL_Rect *trace_dests, bool scrolling);
      
      Time get_time() const;
      bool start_final_leg_p() const;
      void update_state();
      
      BlockList create_count_down_blocks(unsigned count);
      bool delay(unsigned milliseconds);
    
    public:
      Display(unsigned              height,
	      unsigned              width,
	      unsigned              trace_count,
	      const DirectionValue &arrow_direction,
	      unsigned              total_time, /**< In seconds */
	      double                horizon);
      ~Display();
    
      void count_down(unsigned count);
      void start();
      void draw_line(const Line &line);
      Time stop() const;
    
      unsigned get_height() const;
      unsigned get_width() const;
    };
    
  }; /* namespace Resolver */
}; /* namespace Tk */

#endif /* __TK_DISPLAY_H__ */
