/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RESOLVER_TYPES_H__
#define __TK_RESOLVER_TYPES_H__

#include <SDL/SDL.h>

#include "libtkcommon/value.h"
#include "libtkalgorithm/algorithm.h"
#include "libtkresolver/colour.h"

namespace Tk {
  namespace Resolver {
    
    class Offset
    {
    public:
      Tk::Algorithm::Offset offset;
      Colour               *colour;
      Offset(Tk::Algorithm::Offset o, Colour *c = 0)
	: offset(o), colour(c) {}
      Offset(const Offset &other)
	: offset(other.offset), colour(other.colour ? new Colour(*other.colour) : 0) {}
      ~Offset() { if (colour) delete colour; }
    };
   
    typedef std::vector<Offset> Line;
    typedef std::list<Line> Result;
    typedef unsigned short OffsetIndex;
    typedef Uint32 Time; // milliseconds
  
    typedef enum
      {
	Left,
	Right
      } Direction;
    typedef Tk::Value<Direction> DirectionValue;
    
    const short MILLISECONDS_PER_SECOND = 1000;
    
  } /* namespace Resolver */
}; /* namespace Tk */

#endif /* __TK_RESOLVER_TYPES_H__ */
